/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "networkinterface.h"
#include <iostream>
#include <qmessagebox.h>

NetworkInterface::NetworkInterface( QObject *parent, const char *name ) : QObject( parent, name ) {
	proc = new QProcess();
	connect( proc, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout() ) );
	connect( proc, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr() ) );
	connect( proc, SIGNAL( processExited () ), this, SLOT( processHasFinished() ) );
	interfaceTest = false;
	retrieveAllNetworkInterfaces = false;
	retrieveInterfaceIP = false;
	retrieveInterfaceAddress = false;
	retrieveDefaultInterface = false;
	QPtrList<QString>*InterfaceList = new QPtrList<QString>();
	InterfaceList->setAutoDelete( TRUE ); // the list owns the objects
	QString InterfaceIP = "";
	QString InterfaceAddress = "";
	IPforInterface = "";
	tmpInterface = "";
	interfaceExists = false;
	ifconfigbin = "/sbin/ifconfig";
	routebin = "/sbin/route";
}


NetworkInterface::~NetworkInterface() {
	delete proc;
}

bool NetworkInterface::interfaceExist( QString Interface ) {
	if ( !Interface.isEmpty() ) {
		proc->clearArguments();
		proc->addArgument( ifconfigbin );
		proc->addArgument( Interface );

		interfaceTest = true;

		if ( !proc->start() ) {
			std::cerr << "unable to start proc(ifconfig)!" << std::endl;
		}
	}
	while ( interfaceTest == true && proc->isRunning() ) {}
	return interfaceExists;
}


QStringList NetworkInterface::getAllNetworkInterfaces() {

	proc->clearArguments();
	proc->addArgument( ifconfigbin );

	retrieveAllNetworkInterfaces = true;

	if ( !proc->start() ) {
		std::cerr << "unable to start proc(ifconfig)!" << std::endl;
	}

	while ( retrieveAllNetworkInterfaces == true && proc->isRunning() ) {
		sleep( 10 );
	}
	return InterfaceList;
}


QString NetworkInterface::getInterfaceIP( QString Interface ) {
	if ( !Interface.isEmpty() ) {
		proc->clearArguments();
		proc->addArgument( ifconfigbin );
		proc->addArgument( Interface );

		retrieveInterfaceIP = true;

		if ( !proc->start() ) {
			std::cerr << "unable to start proc(ifconfig)!" << std::endl;
		}
	}
	while ( retrieveInterfaceIP == true && proc->isRunning() ) {}
	return InterfaceIP;
}


QString NetworkInterface::getInterfaceAddress( QString IPforInterface ) {
	// TODO fixme
// 	if ( !IPforInterface.isEmpty() ) {
// 		this->IPforInterface = IPforInterface;
// 		QStringList devlist = getAllNetworkInterfaces();
// 
// 		for ( QStringList::Iterator it = devlist.begin(); it != devlist.end(); ++it ) {
// 			tmpInterface = *it;
// 
// 			proc->clearArguments();
// 			proc->addArgument( ifconfigbin );
// 			proc->addArgument( tmpInterface );
// 
// 			retrieveInterfaceAddress = true;
// 
// 			if ( !proc->start() ) {
// 				std::cerr << "unable to start proc(ifconfig)!" << std::endl;
// 			}
// 		}
// 		while ( retrieveInterfaceAddress == true && proc->isRunning() ) {}
// 		return InterfaceAddress;
// 	}
	return "";
}


QString NetworkInterface::getDefaultInterface() {
	proc->clearArguments();
	proc->addArgument( routebin );
	proc->addArgument( "-n" );

	retrieveDefaultInterface = true;

	if ( !proc->start() ) {
		std::cerr << "unable to start proc(route)!" << std::endl;
	}

	while ( retrieveDefaultInterface == true && proc->isRunning() ) {
		sleep( 2 );
	}
	return defaultinterface;

}


void NetworkInterface::readFromStdout() {

	while ( proc->canReadLineStdout() ) {
		QString line = proc->readLineStdout() ;
		//QString line = QString(proc->readStdout());
		

		/*
		example for one interface 
		 
		eth0      Protokoll:Ethernet  Hardware Adresse 00:10:4B:B2:19:00
		          inet Adresse:192.168.0.99  Bcast:192.168.0.255  Maske:255.255.255.0
		          inet6 Adresse: fe80::210:4bff:feb2:1900/64 Gltigkeitsbereich:Verbindung
		          UP BROADCAST NOTRAILERS RUNNING MULTICAST  MTU:1500  Metric:1
		          RX packets:31549 errors:0 dropped:0 overruns:0 frame:0
		          TX packets:34046 errors:0 dropped:0 overruns:0 carrier:0
		          Kollisionen:0 Sendewarteschlangenlnge:1000
		          RX bytes:11308743 (10.7 Mb)  TX bytes:3701511 (3.5 Mb)
		          Interrupt:10 Basisadresse:0xdc00
		*/

		if ( interfaceTest ) {
			if ( line.find( "proto", 0 , FALSE ) != -1 ) {
				interfaceExists = true;
				interfaceTest = false;
			}
		}

		if ( line.find( "proto", 0 , FALSE ) != -1 ) {

			if ( retrieveAllNetworkInterfaces ) {
				QString interface = line.simplifyWhiteSpace().section( ' ', 0, 0 ); // eth0
				InterfaceList.append( interface );
			//	std::cout << "dbg: found interface: " << interface << std::endl;
			}


		}

		if ( retrieveInterfaceIP || retrieveInterfaceAddress ) {
			if ( line.find( "inet a", 0 , FALSE ) != -1 ) {
				QString interfaceIP = line.simplifyWhiteSpace().section( ' ', 1, 1 ).section( ':', 1, 1 ); // eth0
			
				std::cout << "dbg: found interface IP: " << interfaceIP << std::endl;
				if ( retrieveInterfaceIP ) {
					InterfaceIP = interfaceIP;
					retrieveInterfaceIP = false;
				}
				if ( retrieveInterfaceAddress ) {
					if ( line.find( "proto", 0 , FALSE ) != -1 ) {
						if ( IPforInterface == interfaceIP ) {
							InterfaceAddress = interfaceIP;
							std::cout << "dbg: found interface address: " << interfaceIP << std::endl;
							retrieveInterfaceAddress = false;
						}
					}
				}
			}
		}



		if ( retrieveDefaultInterface ) {
			if ( line.find( "UG", 0 , FALSE ) != -1 ) {
				if ( line.find( "0.0.0.0", 0 , FALSE ) != -1 ) {
					defaultinterface = line.simplifyWhiteSpace().section( ' ', -1, -1 );
					retrieveDefaultInterface = false;
				}
			}
		}
	}
}

void NetworkInterface::readFromStderr() {
	while ( proc->canReadLineStderr() ) {
		QString line = proc->readLineStderr() ;

		if ( interfaceTest ) {
			interfaceExists = false;
			interfaceTest = false;
		}


		std::cerr << "dbg err: " << line << std::endl;
	}
}

void NetworkInterface::processHasFinished() {
	//std::cout << "dbg: processHasFinished():" << proc->exitStatus() << std::endl;;
	if ( retrieveAllNetworkInterfaces ) {
		retrieveAllNetworkInterfaces = false;

	}
	if ( retrieveInterfaceIP )
		retrieveInterfaceIP = false;

	// disconnect( proc, SIGNAL( readyReadStdout() ), this, SLOT( readFromStdout() ) );
	// 	disconnect( proc, SIGNAL( readyReadStderr() ), this, SLOT( readFromStderr() ) );
	// 	disconnect( proc, SIGNAL( procExited () ), this, SLOT( procHasFinished() ) );
	//delete proc;
}

#include "networkinterface.moc"
